# Copyright 2018-present Lenovo
# Confidential and Proprietary

import matplotlib
matplotlib.use('Agg')

import numpy as np
import os
import sys
import cv2


# Letrain installation path is fixed
letrain_path = "/opt/letrain"
path_list = [
    os.path.join(letrain_path, "applications"),
    os.path.join(letrain_path, "applications/frcnn")
]
sys.path.extend(path_list)

import tensorflow as tf
from lib.fast_rcnn.config import cfg
from lib.fast_rcnn.test import im_detect
from lib.fast_rcnn.nms_wrapper import nms
from lib.utils.timer import Timer
from lib.networks.factory import get_network
import matplotlib.pyplot as plt
slim = tf.contrib.slim

tf.app.flags.DEFINE_string('demo_net', 'VGGnet_test', 'Network to use [vgg16]')
tf.app.flags.DEFINE_string('model_path', None, 'model_path')
tf.app.flags.DEFINE_string('category_file', '', 'category file')
tf.app.flags.DEFINE_integer('frcnn_num_classes', 21, 'number of classes')
tf.app.flags.DEFINE_string('image_file', None, 'demo image file')
tf.app.flags.DEFINE_string('input_dir', None, 'input dir for inference')
tf.app.flags.DEFINE_string('output_dir', None, 'output dir for inference')
tf.app.flags.DEFINE_string(
    'input_fname_pattern', 'jpg.bmp.png.jpeg',
    'input file pattern, None for all supported formats'
)
tf.app.flags.DEFINE_integer('output_dpi', 72, 'dpi of output image')
tf.app.flags.DEFINE_bool("bbox_reg", False, 'bbox regression')


FLAGS = tf.app.flags.FLAGS

# Force to run inference with CPU mode
cfg.USE_GPU_NMS = False

# modified in _read_category_file();
CLASSES = ('__background__',
           'aeroplane', 'bicycle', 'bird', 'boat',
           'bottle', 'bus', 'car', 'cat', 'chair',
           'cow', 'diningtable', 'dog', 'horse',
           'motorbike', 'person', 'pottedplant',
           'sheep', 'sofa', 'train', 'tvmonitor')


class DetectionDeploySer(object):
    def _read_category_file(self):
        global CLASSES
        if FLAGS.category_file:
            category_file = FLAGS.category_file
        else:
            category_file = os.path.join(FLAGS.model_path, "category.txt")
        _classes_auto = []
        with open(category_file) as f:
            lines = f.readlines()
        for line in lines:
            line = line.lower().strip('\n')
            _classes_auto.append(line)
        CLASSES = tuple(_classes_auto)
        FLAGS.frcnn_num_classes = len(CLASSES)

    def vis_detections(self, im, class_name, dets, ax, thresh=0.5):
        """Example of detect_result:
           [{
               "label": "cat",
               "probability": 0.9456,
               "top": 30,
               "width": 100,
               "height": 200
            },
            ]
        """
        detect_result = []

        """Draw detected bounding boxes."""
        inds = np.where(dets[:, -1] >= thresh)[0]
        if len(inds) == 0:
            return detect_result

        for i in inds:
            bbox = dets[i, :4]
            score = dets[i, -1]

            ax.add_patch(
                plt.Rectangle((bbox[0], bbox[1]),
                              bbox[2] - bbox[0],
                              bbox[3] - bbox[1], fill=False,
                              edgecolor='red', linewidth=3.5)
            )
            ax.text(bbox[0], bbox[1] - 2,
                    '{:s} {:.3f}'.format(class_name, score),
                    bbox=dict(facecolor='blue', alpha=0.5),
                    fontsize=14, color='white')

            detect_result.append(
                {
                    "label": class_name,
                    "probability": round(float(score), 4),
                    "left": int(round(bbox[0])),
                    "top": int(round(bbox[1])),
                    "width": int(round(bbox[2] - bbox[0])),
                    "height": int(round(bbox[3] - bbox[1]))
                }
            )

        ax.set_title(('{} detections with '
                      'p({} | box) >= {:.1f}').format(class_name, class_name,
                                                      thresh),
                     fontsize=14)
        plt.axis('off')
        plt.tight_layout()
        plt.draw()

        return detect_result

    def demo(self, sess, net, image_name):
        """
        Detect object classes in an image
        using pre-computed object proposals.
        """

        # Load the demo image
        # im_file = os.path.join(cfg.DATA_DIR, 'demo', image_name)

        im_file = image_name

        im = cv2.imread(im_file)

        # Detect all object classes and regress object bounds
        timer = Timer()
        timer.tic()
        scores, boxes = im_detect(
            sess, net, im, FLAGS.bbox_reg, bbox_reg=FLAGS.bbox_reg
        )
        timer.toc()
        print('Detection took {:.3f}s for {:d} object proposals'.format(
            timer.total_time, boxes.shape[0]
        ))

        detect_results = []
        frcnn_result = {
            "duration": int(round(1000 * timer.total_time)),
            "data": detect_results
        }

        # Visualize detections for each class
        im = im[:, :, (2, 1, 0)]
        fig, ax = plt.subplots(figsize=(12, 12))
        ax.imshow(im, aspect='equal')

        CONF_THRESH = 0.8
        NMS_THRESH = 0.3
        for cls_ind, cls in enumerate(CLASSES[1:]):
            cls_ind += 1  # because we skipped background
            cls_boxes = boxes[:, 4 * cls_ind:4 * (cls_ind + 1)]
            cls_scores = scores[:, cls_ind]
            dets = np.hstack((cls_boxes,
                              cls_scores[:, np.newaxis])).astype(np.float32)
            keep = nms(dets, NMS_THRESH, 'gpu:0')
            dets = dets[keep, :]
            detect_results.extend(
                self.vis_detections(im, cls, dets, ax, thresh=CONF_THRESH)
            )

        return fig, frcnn_result

    def load_model(self, model_path, parameters):
        cfg.TEST.HAS_RPN = True  # Use RPN for proposals

        # args = parse_args()
        args = FLAGS
        args.model_path = model_path
        if args.model_path == ' ':
            raise IOError('Error: model_path not found.\n')

        # read CLASSES
        self._read_category_file()

        # init session
        self.sess = tf.Session(config=tf.ConfigProto(allow_soft_placement=True))
        # load network
        self.net = get_network(args.demo_net)
        # load model
        # saver = tf.train.Saver(write_version=tf.train.SaverDef.V1)
        # saver.restore(sess, args.model
        variables_to_restore = slim.get_variables_to_restore()
        saver = tf.train.Saver(variables_to_restore)
        ckpt = tf.train.get_checkpoint_state(args.model_path)
        saver.restore(self.sess, ckpt.model_checkpoint_path)

        # sess.run(tf.initialize_all_variables())

        print('\n\nLoaded network {:s}'.format(args.model_path))

        # Warmup on a dummy image
        im = 128 * np.ones((300, 300, 3), dtype=np.uint8)
        for i in range(2):
            _, _ = im_detect(self.sess, self.net, im, bbox_reg=FLAGS.bbox_reg)

    def inference(self, image_path):
        predict_ret = {}
        if image_path:
            print('Demo for {}'.format(image_path))
            predict_fig, predict_ret = self.demo(
                self.sess, self.net, image_path
            )
        return predict_ret


if __name__ == '__main__':
    model_path = '/home/hpcadmin/ai_studio/tasks/'\
        'aba4bbbe-1ae4-11e9-abdd-7cd30ae2c038/zyj_test_0/train_dir'
    image_path = '/home/hpcadmin/mq_test/images/003.jpg'
    response_dir = '/home/hpcadmin/mq_test/response_test_mq'
    parameters = None
    detection_deploy_ser = DetectionDeploySer()
    detection_deploy_ser.load_model(model_path, parameters)
    detection_deploy_ser.inference(image_path)
