# Copyright 2018-present Lenovo
# Confidential and Proprietary

import os
import json
import sys
import time
import tensorflow as tf
from adapter import get_infe_serv


def run_inference(scenario, model_path, model_name, image_file, output_dir):
    inference_service_mgt = get_infe_serv(scenario, model_path, model_name)
    print("Deal with file {}".format(image_file))
    try:
        inf_result = inference_service_mgt.inference(image_file)

        temp_file = os.path.join(
            output_dir, 'temp_file_{}.json'.format(int(time.time()))
        )
        with open(temp_file, 'w') as f:
            inf_result['filename'] = image_file
            f.write(json.dumps(inf_result, indent=4))
            print("{0}".format(inf_result))
        response_file = os.path.join(output_dir, 'result.json')
        os.rename(temp_file, response_file)
        print("Save inference result to {}".format(response_file))
    except Exception as e:
        error_file = os.path.join(output_dir, 'error.json')
        os.mknod(error_file)
        raise e

    sys.stdout.flush()


if __name__ == '__main__':
    tf.app.flags.DEFINE_enum(
        'scenario', 'classification',
        ['classification', 'objectdetection', 'segmentation'],
        "Scenario for this service"
    )
    tf.app.flags.DEFINE_string(
        'lico_model_path', '', "model path, for inference job"
    )
    tf.app.flags.DEFINE_string(
        'lico_image_file', '', "image file to deal, for inference job"
    )
    tf.app.flags.DEFINE_string(
        'lico_output_dir', '', "output dir, for inference job"
    )
    tf.app.flags.DEFINE_string('clas_model_name', '',
                               "model_name for this classification service")
    FLAGS = tf.app.flags.FLAGS

    run_inference(
        FLAGS.scenario,
        FLAGS.lico_model_path,
        FLAGS.clas_model_name,
        FLAGS.lico_image_file,
        FLAGS.lico_output_dir
    )
